@@ -317,6 +317,31 @@ const char * const fv_msg3 = "Format violation: consecutive extended headers fou
 const char * const posix_msg = "This does not look like a POSIX tar archive.";
 const char * const posix_lz_msg = "This does not look like a POSIX tar.lz archive.";
 
+// defined in common.cc
+void xinit_mutex( pthread_mutex_t * const mutex );
+void xinit_cond( pthread_cond_t * const cond );
+void xdestroy_mutex( pthread_mutex_t * const mutex );
+void xdestroy_cond( pthread_cond_t * const cond );
+void xlock( pthread_mutex_t * const mutex );
+void xunlock( pthread_mutex_t * const mutex );
+void xwait( pthread_cond_t * const cond, pthread_mutex_t * const mutex );
+void xsignal( pthread_cond_t * const cond );
+void xbroadcast( pthread_cond_t * const cond );
+unsigned long long parse_octal( const uint8_t * const ptr, const int size );
+int readblock( const int fd, uint8_t * const buf, const int size );
+int writeblock( const int fd, const uint8_t * const buf, const int size );
+
+// defined in common_decode.cc
+class Arg_parser;
+bool block_is_zero( const uint8_t * const buf, const int size );
+bool format_member_name( const Extended & extended, const Tar_header header,
+                         Resizable_buffer & rbuf, const bool long_format );
+bool show_member_name( const Extended & extended, const Tar_header header,
+                       const int vlevel, Resizable_buffer & rbuf );
+bool check_skip_filename( const Arg_parser & parser,
+                          std::vector< char > & name_pending,
+                          const char * const filename, const int filenames );
+
 // defined in create.cc
 enum Solidity { no_solid, bsolid, dsolid, asolid, solid };
 extern int cl_owner;
@@ -339,7 +364,6 @@ int final_exit_status( int retval, const bool show_msg = true );
 unsigned ustar_chksum( const uint8_t * const header );
 bool verify_ustar_chksum( const uint8_t * const header );
 bool has_lz_ext( const std::string & name );
-class Arg_parser;
 int concatenate( const std::string & archive_name, const Arg_parser & parser,
                  const int filenames );
 int encode( const std::string & archive_name, const Arg_parser & parser,
@@ -381,16 +405,6 @@ bool excluded( const char * const filename );
 // defined in extract.cc
 enum Program_mode { m_none, m_append, m_concatenate, m_create, m_delete,
                     m_diff, m_extract, m_list };
-bool block_is_zero( const uint8_t * const buf, const int size );
-bool format_member_name( const Extended & extended, const Tar_header header,
-                         Resizable_buffer & rbuf, const bool long_format );
-bool show_member_name( const Extended & extended, const Tar_header header,
-                       const int vlevel, Resizable_buffer & rbuf );
-bool compare_prefix_dir( const char * const dir, const char * const name );
-bool compare_tslash( const char * const name1, const char * const name2 );
-int readblock( const int fd, uint8_t * const buf, const int size );
-int writeblock( const int fd, const uint8_t * const buf, const int size );
-unsigned long long parse_octal( const uint8_t * const ptr, const int size );
 int decode( const std::string & archive_name, const Arg_parser & parser,
             const int filenames, const int num_workers, const int debug_level,
             const Program_mode program_mode, const bool ignore_ids,
@@ -398,18 +412,6 @@ int decode( const std::string & archive_name, const Arg_parser & parser,
             const bool permissive );
 
 // defined in list_lz.cc
-void xinit_mutex( pthread_mutex_t * const mutex );
-void xinit_cond( pthread_cond_t * const cond );
-void xdestroy_mutex( pthread_mutex_t * const mutex );
-void xdestroy_cond( pthread_cond_t * const cond );
-void xlock( pthread_mutex_t * const mutex );
-void xunlock( pthread_mutex_t * const mutex );
-void xwait( pthread_cond_t * const cond, pthread_mutex_t * const mutex );
-void xsignal( pthread_cond_t * const cond );
-void xbroadcast( pthread_cond_t * const cond );
-bool check_skip_filename( const Arg_parser & parser,
-                          std::vector< char > & name_pending,
-                          const char * const filename, const int filenames );
 struct LZ_Decoder;
 int archive_read_lz( LZ_Decoder * const decoder, const int infd,
                      long long & file_pos, const long long member_end,
